@@ -4,7 +4,14 @@ puts [exec ./build-dyn.sh]
 
 load ./tuapi.so
 
-::tuapi::syscall::tsmf_start_svc blah /bin/true /tmp/logfile [list PATH=/bin] / 022 0 0 10
+puts [::tuapi::scan_and_load_kernel_modules]
+
+#foreach x [list AS CORE CPU DATA FSIZE LOCKS MEMLOCK MSGQUEUE NICE NOFILE OFILE NPROC RSS RTPRIO RTTIME SIGPENDING STACK] {
+#	puts "\t\tcase [format 0x%xLU [::tuapi::internal::hash $x]]: /* $x */"
+#	puts "\t\t\tresource_id = RLIMIT_$x;"
+#	puts "\t\t\tbreak;"
+#}
+#exit
 
 foreach iface [tuapi::syscall::ifconfig] {
 #lo0:2: flags=2001000849<UP,LOOPBACK,RUNNING,MULTICAST,IPv4,VIRTUAL> mtu 8232 index 1
