@@ -1,12 +1,17 @@
 #define _LINUX_SOURCE 1
+#include <sys/resource.h>
 #include <sys/syscall.h>
+#include <sys/termios.h>
 #include <netinet/in.h>
+#include <sys/reboot.h>
 #include <arpa/inet.h>
 #include <sys/socket.h>
 #include <sys/select.h>
 #include <sys/mount.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
+#include <sys/time.h>
+#include <sys/klog.h>
 #include <sys/swap.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -43,6 +48,24 @@
 #ifndef MS_MOVE
 #define MS_MOVE 8192
 #endif
+#ifndef SYSLOG_ACTION_CLOSE
+#define SYSLOG_ACTION_CLOSE 0
+#endif
+#ifndef SYSLOG_ACTION_OPEN
+#define SYSLOG_ACTION_OPEN 1
+#endif
+#ifndef SYSLOG_ACTION_READ_ALL
+#define SYSLOG_ACTION_READ_ALL 3
+#endif
+#ifndef SYSLOG_ACTION_CLEAR
+#define SYSLOG_ACTION_CLEAR 5
+#endif
+#ifndef SYSLOG_ACTION_CONSOLE_OFF
+#define SYSLOG_ACTION_CONSOLE_OFF 6
+#endif
+#ifndef SYSLOG_ACTION_CONSOLE_ON
+#define SYSLOG_ACTION_CONSOLE_ON 7
+#endif
 
 /* Simple macros */
 #ifndef MAX
@@ -61,6 +84,11 @@ static int init_module(void *val, unsigned long len, const char *args) {
 	return(syscall(SYS_init_module, val, len, args));
 }
 #endif
+#ifdef SYS_delete_module
+static int delete_module(const char *name, int flags) {
+	return(syscall(SYS_delete_module, name, flags));
+}
+#endif
 #ifdef SYS_pivot_root
 static int pivot_root(const char *new_root, const char *put_old) {
 	return(syscall(SYS_pivot_root, new_root, put_old));
@@ -430,11 +458,12 @@ static int tuapi_insmod(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CO
 	Tcl_Channel fd;
 	Tcl_Obj *module_filename, *module_data;
 	void *module_data_val;
+	const char *module_opts;
 	int module_data_len;
 	int read_ret, chk_ret;
 
-	if (objc < 2) {
-		Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \"tuapi::syscall::insmod filename ?args ...?\"", -1));
+	if (objc < 2 || objc > 3) {
+		Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \"tuapi::syscall::insmod filename ?args?\"", -1));
 
 		return(TCL_ERROR);
 	}
@@ -467,7 +496,13 @@ static int tuapi_insmod(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CO
 
 	module_data_val = Tcl_GetByteArrayFromObj(module_data, &module_data_len);
 
-	chk_ret = init_module(module_data_val, module_data_len, "");
+	if (objc == 3) {
+		module_opts = Tcl_GetString(objv[2]);
+	} else {
+		module_opts = "";
+	}
+
+	chk_ret = init_module(module_data_val, module_data_len, module_opts);
 	if (chk_ret != 0) {
 		Tcl_SetObjResult(interp, Tcl_NewStringObj(strerror(errno), -1));
 
@@ -478,9 +513,28 @@ static int tuapi_insmod(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CO
 }
 
 static int tuapi_rmmod(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
-	Tcl_SetObjResult(interp, Tcl_NewStringObj("not implemented", -1));
+	char *module;
+	int idx;
+	int delete_module_ret;
 
-	return(TCL_ERROR);
+	if (objc < 2) {
+		Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \"tuapi::syscall::rmmod module ?module...?", -1));
+
+		return(TCL_ERROR);
+	}
+
+	for (idx = 1; idx < objc; idx++) {
+		module = Tcl_GetString(objv[idx]);
+
+		delete_module_ret = delete_module(module, O_NONBLOCK);
+		if (delete_module_ret != 0) {
+			Tcl_SetObjResult(interp, Tcl_ObjPrintf("unable to remove \"%s\": %s", module, strerror(errno)));
+
+			return(TCL_ERROR);
+		}
+	}
+
+	return(TCL_OK);
 }
 
 static int tuapi_lsmod(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
@@ -766,6 +820,87 @@ static int tuapi_kill(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONS
 	return(TCL_OK);
 }
 
+static int tuapi_reboot(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
+	Tcl_Obj *cmd_obj;
+	int cmd;
+	int reboot_ret;
+
+	if (objc == 2) {
+		cmd_obj = objv[1];
+
+		switch (tuapi_internal_simplehash_obj(cmd_obj)) {
+			case 0x2be1946: /* LINUX_REBOOT_CMD_CAD_OFF */
+			case 0x666e6344: /* RB_DISABLE_CAD */
+			case 0x9e3ce644: /* DISABLE_CAD */
+				cmd = RB_DISABLE_CAD;
+
+				break;
+			case 0xe8057c4e: /* LINUX_REBOOT_CMD_CAD_ON */
+			case 0xf8dc444: /* RB_ENABLE_CAD */
+			case 0x1a7d6144: /* ENABLE_CAD */
+				cmd = RB_ENABLE_CAD;
+
+				break;
+			case 0x95bfa454: /* LINUX_REBOOT_CMD_HALT */
+			case 0x3210da4d: /* RB_HALT_SYSTEM */
+			case 0xca425f4d: /* HALT_SYSTEM */
+			case 0x9106654: /* HALT */
+				cmd = RB_HALT_SYSTEM;
+
+				break;
+			case 0xdb55d8c6: /* LINUX_REBOOT_CMD_POWER_OFF */
+			case 0xf07700c6: /* RB_POWER_OFF */
+			case 0x645ce1c6: /* POWER_OFF */
+				cmd = 0x4321fedc;
+
+				break;
+			case 0x73ff83d4: /* LINUX_REBOOT_CMD_RESTART */
+			case 0x3cd0e254: /* RB_AUTOBOOT */
+			case 0xb9f8b5d4: /* AUTOBOOT */
+			case 0x3a357fd4: /* RESTART */
+			case 0x5852add4: /* REBOOT */
+				cmd = RB_AUTOBOOT;
+
+				break;
+			default:
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("unknown or ambiguous subcommand: must be DISABLE_CAD, ENABLE_CAD, HALT, POWER_OFF, or RESTART", -1));
+
+				return(TCL_ERROR);
+		}
+	} else if (objc == 1) {
+		cmd = RB_AUTOBOOT;
+	} else {
+		Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \"::tuapi::syscall::reboot ?command?", -1));
+
+		return(TCL_ERROR);
+	}
+
+	switch (cmd) {
+		case RB_ENABLE_CAD:
+		case RB_DISABLE_CAD:
+			/* No need to sync for these operations */
+			break;
+		default:
+			sync();
+			break;
+	}
+
+	reboot_ret = reboot(cmd);
+	if (reboot_ret != 0) {
+		Tcl_SetObjResult(interp, Tcl_NewStringObj(strerror(errno), -1));
+
+		return(TCL_ERROR);
+	}
+
+	return(TCL_OK);
+}
+
+static int tuapi_eject(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
+	Tcl_SetObjResult(interp, Tcl_NewStringObj("not implemented", -1));
+
+	return(TCL_ERROR);
+}
+
 static int tuapi_ps(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
 	Tcl_SetObjResult(interp, Tcl_NewStringObj("not implemented", -1));
 
@@ -1154,9 +1289,11 @@ static int tuapi_ifconfig_info(ClientData cd, Tcl_Interp *interp, int objc, Tcl_
 			case ARPHRD_HWX25:
 				link_encap = "hwx25";
 				break;
+#ifdef ARPHRD_CAN
 			case ARPHRD_CAN:
 				link_encap = "can";
 				break;
+#endif
 			case ARPHRD_PPP:
 				link_encap = "ppp";
 				break;
@@ -1879,6 +2016,401 @@ static int tuapi_vconfig(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *C
 	return(retval);
 }
 
+static int tuapi_stty(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
+	Tcl_Obj *obj, *retobj = NULL;
+	struct termios terminal_information;
+	struct winsize terminal_size;
+	unsigned long obj_hash;
+	int fd, idx;
+	int ioctl_ret;
+	int retval = TCL_OK;
+
+	fd = STDIN_FILENO;
+
+	for (idx = 1; idx < objc; idx++) {
+		obj = objv[idx];
+		obj_hash = tuapi_internal_simplehash_obj(obj);
+
+		switch (obj_hash) {
+			case 0xe7a7d65: /* size */
+				ioctl_ret = ioctl(fd, TIOCGWINSZ, &terminal_size);
+				if (ioctl_ret != 0) {
+					Tcl_SetObjResult(interp, Tcl_NewStringObj("ioctl failed", -1));
+
+					return(TCL_ERROR);
+				}
+
+				if (retobj == NULL) {
+					retobj = Tcl_NewObj();
+				}
+
+				Tcl_ListObjAppendElement(interp, retobj, Tcl_NewLongObj(terminal_size.ws_row));
+				Tcl_ListObjAppendElement(interp, retobj, Tcl_NewLongObj(terminal_size.ws_col));
+
+				break;
+			case 0x5bcb0f7: /* -raw */
+			case 0x1cb0f7: /* raw */
+			case 0xdcb8f56f: /* -echo */
+			case 0xcb8f46f: /* echo */
+				ioctl_ret = ioctl(fd, TCGETS, &terminal_information);
+				if (ioctl_ret != 0) {
+					Tcl_SetObjResult(interp, Tcl_NewStringObj("ioctl failed", -1));
+
+					return(TCL_ERROR);
+				}
+
+				switch (obj_hash) {
+					case 0x5bcb0f7: /* -raw */
+						terminal_information.c_iflag |= BRKINT | IGNPAR | ISTRIP | ICRNL | IXON;
+						terminal_information.c_oflag |= OPOST;
+						terminal_information.c_lflag |= ISIG | ICANON;
+#if VMIN == VEOF
+						terminal_information.c_cc[VEOF] = CEOF;
+#endif
+#if VTIME == VEOL
+						terminal_information.c_cc[VEOL] = CEOL;
+#endif
+						break;
+					case 0x1cb0f7: /* raw */
+						terminal_information.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
+						terminal_information.c_oflag &= ~OPOST;
+						terminal_information.c_lflag &= ~(ISIG | ICANON);
+						terminal_information.c_cc[VMIN] = 1;
+						terminal_information.c_cc[VTIME] = 0;
+						break;
+					case 0xdcb8f56f: /* -echo */
+						terminal_information.c_lflag &= ~ECHO;
+						break;
+					case 0xcb8f46f: /* echo */
+						terminal_information.c_lflag |= ECHO;
+						break;
+				}
+
+				ioctl_ret = ioctl(fd, TCSETS, &terminal_information);
+				if (ioctl_ret != 0) {
+					Tcl_SetObjResult(interp, Tcl_NewStringObj("ioctl failed", -1));
+
+					return(TCL_ERROR);
+				}
+
+				break;
+			default:
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("subcommand not implemented", -1));
+				return(TCL_ERROR);
+		}
+	}
+
+	if (retobj != NULL) {
+		Tcl_SetObjResult(interp, retobj);
+	}
+
+	return(retval);
+}
+
+static int tuapi_rlimit(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
+	Tcl_Obj *operation_obj, *resource_id_obj, *resource_val_item_obj, *resource_val_itemval_obj, *ret_obj;
+	struct rlimit resource_val;
+	Tcl_WideInt resource_val_item;
+	int resource_id;
+	int rlimit_ret, tcl_ret;
+
+	if (objc < 3) {
+		Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \"::tuapi::syscall::rlimit operation resource ?value?\"", -1));
+
+		return(TCL_ERROR);
+	}
+
+	operation_obj = objv[1];
+	resource_id_obj = objv[2];
+
+	switch (tuapi_internal_simplehash_obj(resource_id_obj)) {
+		case 0x20d3LU: /* AS */
+			resource_id = RLIMIT_AS;
+			break;
+		case 0x873e945LU: /* CORE */
+			resource_id = RLIMIT_CORE;
+			break;
+		case 0x10e855LU: /* CPU */
+			resource_id = RLIMIT_CPU;
+			break;
+		case 0x8906a41LU: /* DATA */
+			resource_id = RLIMIT_DATA;
+			break;
+		case 0x6a726f45LU: /* FSIZE */
+			resource_id = RLIMIT_FSIZE;
+			break;
+#ifdef RLIMIT_LOCKS
+		case 0xc9f0e7d3LU: /* LOCKS */
+			resource_id = RLIMIT_LOCKS;
+			break;
+#endif
+		case 0xd908f7cbLU: /* MEMLOCK */
+			resource_id = RLIMIT_MEMLOCK;
+			break;
+#ifdef RLIMIT_MSGQUEUE
+		case 0x57167445LU: /* MSGQUEUE */
+			resource_id = RLIMIT_MSGQUEUE;
+			break;
+#endif
+		case 0x9d261c5LU: /* NICE */
+			resource_id = RLIMIT_NICE;
+			break;
+		case 0xf8d35c45LU: /* NOFILE */
+		case 0xf8d26445LU: /* OFILE */
+			resource_id = RLIMIT_NOFILE;
+			break;
+		case 0xea14a5c3LU: /* NPROC */
+			resource_id = RLIMIT_NPROC;
+			break;
+		case 0x14a9d3LU: /* RSS */
+			resource_id = RLIMIT_RSS;
+			break;
+#ifdef RLIMIT_RTPRIO
+		case 0x4a15ee4fLU: /* RTPRIO */
+			resource_id = RLIMIT_RTPRIO;
+			break;
+#endif
+#ifdef RLIMIT_RTTIME
+		case 0x4a932c45LU: /* RTTIME */
+			resource_id = RLIMIT_RTTIME;
+			break;
+#endif
+#ifdef RLIMIT_SIGPENDING
+		case 0x2f390347LU: /* SIGPENDING */
+			resource_id = RLIMIT_SIGPENDING;
+			break;
+#endif
+		case 0x3a90634bLU: /* STACK */
+			resource_id = RLIMIT_STACK;
+			break;
+		default:
+			Tcl_SetObjResult(interp, Tcl_NewStringObj("invalid resource", -1));
+
+			return(TCL_ERROR);
+	}
+
+	if (strcmp(Tcl_GetString(operation_obj), "get") == 0) {
+		if (objc != 3) {
+			Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \"::tuapi::syscall::rlimit get resource\"", -1));
+
+			return(TCL_ERROR);
+		}
+
+		rlimit_ret = getrlimit(resource_id, &resource_val);
+		if (rlimit_ret != 0) {
+			Tcl_SetObjResult(interp, Tcl_NewStringObj("getrlimit() failed", -1));
+
+			return(TCL_ERROR);
+		}
+
+		ret_obj = Tcl_NewObj();
+		Tcl_ListObjAppendElement(interp, ret_obj, Tcl_NewStringObj("soft", -1));
+		Tcl_ListObjAppendElement(interp, ret_obj, Tcl_NewWideIntObj(resource_val.rlim_cur));
+		Tcl_ListObjAppendElement(interp, ret_obj, Tcl_NewStringObj("hard", -1));
+		Tcl_ListObjAppendElement(interp, ret_obj, Tcl_NewWideIntObj(resource_val.rlim_max));
+
+		Tcl_SetObjResult(interp, ret_obj);
+
+		return(TCL_OK);
+	}
+
+	if (strcmp(Tcl_GetString(operation_obj), "set") == 0) {
+		if (objc != 4) {
+			Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \"::tuapi::syscall::rlimit set resource value\"", -1));
+
+			return(TCL_ERROR);
+		}
+
+		resource_val_item_obj = objv[3];
+
+		/* Determine if we were asked to set to a simple value, in which case set both hard and soft limits */
+		tcl_ret = Tcl_GetWideIntFromObj(NULL, resource_val_item_obj, &resource_val_item);
+		if (tcl_ret == TCL_OK) {
+			resource_val.rlim_cur = resource_val_item;
+			resource_val.rlim_max = resource_val_item;
+		} else {
+			tcl_ret = Tcl_DictObjGet(NULL, resource_val_item_obj, Tcl_NewStringObj("soft", -1), &resource_val_itemval_obj);
+			if (tcl_ret != TCL_OK) {
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("invalid request", -1));
+
+				return(TCL_ERROR);
+			}
+
+			tcl_ret = Tcl_GetWideIntFromObj(NULL, resource_val_itemval_obj, &resource_val_item);
+			if (tcl_ret != TCL_OK) {
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("invalid request", -1));
+
+				return(TCL_ERROR);
+			}
+
+			resource_val.rlim_cur = resource_val_item;
+
+			tcl_ret = Tcl_DictObjGet(NULL, resource_val_item_obj, Tcl_NewStringObj("hard", -1), &resource_val_itemval_obj);
+			if (tcl_ret != TCL_OK) {
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("invalid request", -1));
+
+				return(TCL_ERROR);
+			}
+
+			tcl_ret = Tcl_GetWideIntFromObj(NULL, resource_val_itemval_obj, &resource_val_item);
+			if (tcl_ret != TCL_OK) {
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("invalid request", -1));
+
+				return(TCL_ERROR);
+			}
+
+			resource_val.rlim_max = resource_val_item;
+		}
+
+		rlimit_ret = setrlimit(resource_id, &resource_val);
+		if (rlimit_ret != 0) {
+			Tcl_SetObjResult(interp, Tcl_NewStringObj("setrlimit() failed", -1));
+
+			return(TCL_ERROR);
+		}
+
+		return(TCL_OK);
+	}
+
+	Tcl_SetObjResult(interp, Tcl_NewStringObj("invalid operation", -1));
+
+	return(TCL_ERROR);
+}
+
+static int tuapi_klogctl(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
+	Tcl_Obj *operation_obj;
+	char *buf;
+	int buflen;
+	int klog_ret;
+
+	if (objc < 2) {
+		Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \"::tuapi::syscall::klogctl operation ...\"", -1));
+
+		return(TCL_ERROR);
+	}
+
+	klog_ret = klogctl(SYSLOG_ACTION_OPEN, NULL, 0);
+	if (klog_ret != 0) {
+		Tcl_SetObjResult(interp, Tcl_NewStringObj("klogctl(SYSLOG_ACTION_OPEN, ...) failed", -1));
+		return(TCL_ERROR);
+	}
+
+	operation_obj = objv[1];
+	switch (tuapi_internal_simplehash_obj(operation_obj)) {
+		case 0xe5970e4LU: /* read */
+			buflen = 256 * 1024;
+			buf = malloc(buflen);
+			if (buf == NULL) {
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("malloc failed !", -1));
+
+				return(TCL_ERROR);
+			}
+
+			klog_ret = klogctl(SYSLOG_ACTION_READ_ALL, buf, buflen);
+			if (klog_ret == -1) {
+				free(buf);
+
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("klogctl(SYSLOG_ACTION_READ_ALL, ...) failed", -1));
+
+				return(TCL_ERROR);
+			}
+
+			Tcl_SetObjResult(interp, Tcl_NewByteArrayObj((unsigned char *) buf, klog_ret));
+
+			free(buf);
+
+			return(TCL_OK);
+		case 0x3d9973f2LU: /* clear */
+			klog_ret = klogctl(SYSLOG_ACTION_CLEAR, NULL, 0);
+			if (klog_ret == -1) {
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("klogctl(SYSLOG_ACTION_CLEAR, ...) failed", -1));
+
+				return(TCL_ERROR);
+			}
+
+			return(TCL_OK);
+		case 0x225c336eLU: /* console_on */
+			klog_ret = klogctl(SYSLOG_ACTION_CONSOLE_ON, NULL, 0);
+			if (klog_ret == -1) {
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("klogctl(SYSLOG_ACTION_CONSOLE_ON, ...) failed", -1));
+
+				return(TCL_ERROR);
+			}
+
+			return(TCL_OK);
+		case 0x2e19bbe6LU: /* console_off */
+			klog_ret = klogctl(SYSLOG_ACTION_CONSOLE_OFF, NULL, 0);
+			if (klog_ret == -1) {
+				Tcl_SetObjResult(interp, Tcl_NewStringObj("klogctl(SYSLOG_ACTION_CONSOLE_OFF, ...) failed", -1));
+
+				return(TCL_ERROR);
+			}
+
+			return(TCL_OK);
+	}
+
+	Tcl_SetObjResult(interp, Tcl_NewStringObj("invalid subcommand", -1));
+
+	return(TCL_ERROR);
+}
+
+static int tuapi_waitpid(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
+	pid_t child;
+	int status;
+
+	child = waitpid(-1, &status, WNOHANG);
+	if (child < 0) {
+		if (errno != ECHILD) {
+			Tcl_SetObjResult(interp, Tcl_NewStringObj(strerror(errno), -1));
+
+			return(TCL_ERROR);
+		} else {
+			child = 0;
+		}
+	}
+
+	if (child != 0) {
+		Tcl_SetObjResult(interp, Tcl_NewWideIntObj(child));
+	}
+
+	return(TCL_OK);
+}
+
+static int tuapi_settimeofday(ClientData cd, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
+	struct timeval tv;
+	Tcl_WideInt tv_sec_val, tv_usec_val;
+	int settimeofday_ret, tcl_ret;
+
+	if (objc != 3) {
+		Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args: should be \"::tuapi::syscall::settimeofday seconds microseconds\"", -1));
+
+		return(TCL_ERROR);
+	}
+
+	tcl_ret = Tcl_GetWideIntFromObj(interp, objv[1], &tv_sec_val);
+	if (tcl_ret != TCL_OK) {
+		return(tcl_ret);
+	}
+
+	tcl_ret = Tcl_GetWideIntFromObj(interp, objv[2], &tv_usec_val);
+	if (tcl_ret != TCL_OK) {
+		return(tcl_ret);
+	}
+
+	tv.tv_sec = tv_sec_val;
+	tv.tv_usec = tv_usec_val; 
+
+	settimeofday_ret = settimeofday(&tv, NULL);
+
+	if (settimeofday_ret < 0) {
+		Tcl_SetObjResult(interp, Tcl_NewStringObj(strerror(errno), -1));
+
+		return(TCL_ERROR);
+	}
+
+	return(TCL_OK);
+}
+
 #ifndef DISABLE_UNIX_SOCKETS
 struct tuapi_socket_unix__chan_id {
 	int fd;
@@ -2380,11 +2912,14 @@ static int tuapi_tsmf_start_svc(ClientData cd, Tcl_Interp *interp, int objc, Tcl
 
 	if (child != 0) {
 		/* 4.parent. Get PGID from child */
-		/* 4.parent.a. Close write end of pipe -- we are read-only */
+		/* 4.parent.a. Open log file */
+		log_fd = open(logfile, O_WRONLY | O_CREAT | O_APPEND, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+		/* 4.parent.b. Close write end of pipe -- we are read-only */
 		close(fds[1]);
 		fd = fds[0];
 
-		/* 4.parent.b. Read process group ID of child from pipe */
+		/* 4.parent.c. Read process group ID of child from pipe */
 		select_timeout.tv_sec = timeout_val;
 		select_timeout.tv_usec = 0;
 
@@ -2401,6 +2936,15 @@ static int tuapi_tsmf_start_svc(ClientData cd, Tcl_Interp *interp, int objc, Tcl
 
 			Tcl_SetObjResult(interp, Tcl_NewStringObj("timeout", -1));
 
+			currtime = time(NULL);
+			strftime(logmsg, sizeof(logmsg), "[ %b %e %H:%M:%S ", localtime(&currtime));
+			write(log_fd, logmsg, strlen(logmsg));
+
+			snprintf(logmsg, sizeof(logmsg), "Method \"start\" timed out after %i seconds ]\n", (int) timeout_val);
+			write(log_fd, logmsg, strlen(logmsg));
+
+			close(log_fd);
+
 			return(TCL_ERROR);
 		}
 
@@ -2408,26 +2952,53 @@ static int tuapi_tsmf_start_svc(ClientData cd, Tcl_Interp *interp, int objc, Tcl
 			read_ret = read(fd, &child_pgid, sizeof(child_pgid));
 		}
 
-		/* 4.parent.c. Close read end of pipe */
+		/* 4.parent.d. Close read end of pipe */
 		close(fd);
 
-		/* 4.parent.d. Verify read was meaningful */
+		/* 4.parent.e. Verify read was meaningful */
 		if (read_ret != sizeof(child_pgid)) {
 			Tcl_SetObjResult(interp, Tcl_NewStringObj("failed to communicate with started service", -1));
 
+			currtime = time(NULL);
+			strftime(logmsg, sizeof(logmsg), "[ %b %e %H:%M:%S ", localtime(&currtime));
+			write(log_fd, logmsg, strlen(logmsg));
+
+			snprintf(logmsg, sizeof(logmsg), "Method \"start\" failed: communication with started service broken ]\n");
+			write(log_fd, logmsg, strlen(logmsg));
+
+			close(log_fd);
+
 			return(TCL_ERROR);
 		}
 
-		/* 4.parent.e. If the PGID given is actually an error, return error */
+		/* 4.parent.f. If the PGID given is actually an error, return error */
 		if (child_pgid == -1) {
 			Tcl_SetObjResult(interp, Tcl_NewStringObj("service failed to start", -1));
 
+			currtime = time(NULL);
+			strftime(logmsg, sizeof(logmsg), "[ %b %e %H:%M:%S ", localtime(&currtime));
+			write(log_fd, logmsg, strlen(logmsg));
+
+			snprintf(logmsg, sizeof(logmsg), "Method \"start\" failed ]\n");
+			write(log_fd, logmsg, strlen(logmsg));
+
+			close(log_fd);
+
 			return(TCL_ERROR);
 		}
 
-		/* 4.parent.f. Return PGID to Tcl */
+		/* 4.parent.g. Return PGID to Tcl */
 		Tcl_SetObjResult(interp, Tcl_NewWideIntObj((Tcl_WideInt) child_pgid));
 
+		currtime = time(NULL);
+		strftime(logmsg, sizeof(logmsg), "[ %b %e %H:%M:%S ", localtime(&currtime));
+		write(log_fd, logmsg, strlen(logmsg));
+
+		snprintf(logmsg, sizeof(logmsg), "Method \"start\" completed, process group = %lu ]\n", (unsigned long) child_pgid);
+		write(log_fd, logmsg, strlen(logmsg));
+
+		close(log_fd);
+
 		return(TCL_OK);
 	}
 
@@ -2572,9 +3143,12 @@ int Tuapi_Init(Tcl_Interp *interp) {
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::lsmod", tuapi_lsmod, NULL, NULL);
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::hostname", tuapi_hostname, NULL, NULL);
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::domainname", tuapi_domainname, NULL, NULL);
+	Tcl_CreateObjCommand(interp, "::tuapi::syscall::klogctl", tuapi_klogctl, NULL, NULL);
+	Tcl_CreateObjCommand(interp, "::tuapi::syscall::settimeofday", tuapi_settimeofday, NULL, NULL);
 
 	/* Block or char device related commands */
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::losetup", tuapi_losetup, NULL, NULL);
+	Tcl_CreateObjCommand(interp, "::tuapi::syscall::eject", tuapi_eject, NULL, NULL);
 
 	/* Filesystem related commands */
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::mount", tuapi_mount, NULL, NULL);
@@ -2588,8 +3162,11 @@ int Tuapi_Init(Tcl_Interp *interp) {
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::chroot", tuapi_chroot, NULL, NULL);
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::pivot_root", tuapi_pivot_root, NULL, NULL);
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::kill", tuapi_kill, NULL, NULL);
+	Tcl_CreateObjCommand(interp, "::tuapi::syscall::waitpid", tuapi_waitpid, NULL, NULL);
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::ps", tuapi_ps, NULL, NULL);
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::execve", tuapi_execve, NULL, NULL);
+	Tcl_CreateObjCommand(interp, "::tuapi::syscall::rlimit", tuapi_rlimit, NULL, NULL);
+	Tcl_CreateObjCommand(interp, "::tuapi::syscall::reboot", tuapi_reboot, NULL, NULL);
 
 	/* Network related commands */
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::ifconfig", tuapi_ifconfig, NULL, NULL);
@@ -2597,6 +3174,9 @@ int Tuapi_Init(Tcl_Interp *interp) {
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::brctl", tuapi_brctl, NULL, NULL);
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::vconfig", tuapi_vconfig, NULL, NULL);
 
+	/* Terminal related commands */
+	Tcl_CreateObjCommand(interp, "::tuapi::syscall::stty", tuapi_stty, NULL, NULL);
+
 	/* Needed commands for basic services Tcl lacks */
 	Tcl_CreateObjCommand(interp, "::tuapi::syscall::socket_unix", tuapi_socket_unix, NULL, NULL);
 
@@ -2618,7 +3198,7 @@ int Tuapi_Init(Tcl_Interp *interp) {
 #include "tuapi.tcl.h" 
 	);
 
-	Tcl_PkgProvide(interp, "tuapi", "0.4");
+	Tcl_PkgProvide(interp, "tuapi", "0.6");
 
 	return(TCL_OK);
 }
